@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 4/17/2019.
-//  Copyright (c) 2020-2023 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2020-2024 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -535,7 +535,7 @@ void QtSLiMVariableBrowser::itemExpanded(QTreeWidgetItem *item)
             // we used to display zero-length property values for zero-length object vectors, but don't any more
             int display_index = (element_index != -1) ? element_index : 0;
             EidosValue_Object *eidos_object_vector = static_cast<EidosValue_Object *>(eidos_value);
-            EidosObject *eidos_object = eidos_object_vector->ObjectElementAtIndex(display_index, nullptr);
+            EidosObject *eidos_object = eidos_object_vector->ObjectElementAtIndex_NOCAST(display_index, nullptr);
 			const EidosClass *object_class = eidos_object->Class();
 			const std::vector<EidosPropertySignature_CSP> *properties = object_class->Properties();
 			size_t propertyCount = properties->size();
