@@ -556,6 +556,7 @@ EidosValue_SP Community::ExecuteInstanceMethod(EidosGlobalStringID p_method_id,
 		case gID_scriptBlocksWithIDs:			return ExecuteMethod_scriptBlocksWithIDs(p_method_id, p_arguments, p_interpreter);
 		case gID_speciesWithIDs:				return ExecuteMethod_speciesWithIDs(p_method_id, p_arguments, p_interpreter);
 		case gID_subpopulationsWithIDs:			return ExecuteMethod_subpopulationsWithIDs(p_method_id, p_arguments, p_interpreter);
+		case gID_subpopulationsWithNames:		return ExecuteMethod_subpopulationsWithNames(p_method_id, p_arguments, p_interpreter);
 		case gID_outputUsage:					return ExecuteMethod_outputUsage(p_method_id, p_arguments, p_interpreter);
 		case gID_registerFirstEvent:
 		case gID_registerEarlyEvent:
@@ -658,9 +659,6 @@ EidosValue_SP Community::ExecuteMethod_createLogFile(EidosGlobalStringID p_metho
 EidosValue_SP Community::ExecuteMethod_estimatedLastTick(EidosGlobalStringID p_method_id, const std::vector<EidosValue_SP> &p_arguments, EidosInterpreter &p_interpreter)
 {
 #pragma unused (p_method_id, p_arguments, p_interpreter)
-	if (!tick_ranges_available_)
-		EIDOS_TERMINATION << "ERROR (Community::ExecuteMethod_estimatedLastTick): estimatedLastTick() cannot be called until script block tick ranges have been evaluated, after initialize() callbacks have finished executing." << EidosTerminate();
-	
 	slim_tick_t last_tick = EstimatedLastTick();
 	
 	return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int(last_tick));
@@ -872,6 +870,30 @@ EidosValue_SP Community::ExecuteMethod_subpopulationsWithIDs(EidosGlobalStringID
 	return EidosValue_SP(vec);
 }
 
+//	*********************	– (object<Subpopulation>)subpopulationsWithNames(string names)
+//
+EidosValue_SP Community::ExecuteMethod_subpopulationsWithNames(EidosGlobalStringID p_method_id, const std::vector<EidosValue_SP> &p_arguments, EidosInterpreter &p_interpreter)
+{
+#pragma unused (p_method_id, p_interpreter)
+	EidosValue *names_value = p_arguments[0].get();
+	int names_count = names_value->Count();
+	const std::string *names_data = names_value->StringData();
+	EidosValue_Object *vec = (new (gEidosValuePool->AllocateChunk()) EidosValue_Object(gSLiM_Subpopulation_Class))->resize_no_initialize_RR(names_count);
+	
+	for (int name_index = 0; name_index < names_count; name_index++)
+	{
+		const std::string &name = names_data[name_index];
+		Subpopulation *object = SubpopulationWithName(name);
+		
+		if (!object)
+			EIDOS_TERMINATION << "ERROR (Community::ExecuteMethod_subpopulationsWithNames): subpopulationsWithNames() did not find a subpopulation with name " << name << "." << EidosTerminate();
+		
+		vec->set_object_element_no_check_NORR(object, name_index);
+	}
+	
+	return EidosValue_SP(vec);
+}
+
 //	*********************	– (void)outputUsage(void)
 //
 EidosValue_SP Community::ExecuteMethod_outputUsage(EidosGlobalStringID p_method_id, const std::vector<EidosValue_SP> &p_arguments, EidosInterpreter &p_interpreter)
@@ -1178,6 +1200,7 @@ EidosValue_SP Community::ExecuteMethod_rescheduleScriptBlock(EidosGlobalStringID
 		
 		CheckScheduling(start, stage);
 		
+		block->tick_range_evaluated_ = true;
 		block->tick_range_is_sequence_ = true;
 		block->tick_start_ = start;
 		block->tick_end_ = end;
@@ -1206,6 +1229,7 @@ EidosValue_SP Community::ExecuteMethod_rescheduleScriptBlock(EidosGlobalStringID
 		if (tick_count == 0)
 		{
 			// set to run in no ticks; we do this with an empty set
+			block->tick_range_evaluated_ = true;
 			block->tick_range_is_sequence_ = false;
 			block->tick_set_.clear();
 		}
@@ -1236,6 +1260,7 @@ EidosValue_SP Community::ExecuteMethod_rescheduleScriptBlock(EidosGlobalStringID
 			
 			if (is_sequential)
 			{
+				block->tick_range_evaluated_ = true;
 				block->tick_range_is_sequence_ = true;
 				block->tick_start_ = (slim_tick_t)first_value;
 				block->tick_end_ = (slim_tick_t)prev_value;
@@ -1248,6 +1273,7 @@ EidosValue_SP Community::ExecuteMethod_rescheduleScriptBlock(EidosGlobalStringID
 				std::unordered_set<slim_tick_t> &set = block->tick_set_;
 				slim_tick_t min_tick = SLIM_MAX_TICK;
 				
+				block->tick_range_evaluated_ = true;
 				block->tick_range_is_sequence_ = false;
 				set.clear();
 				
@@ -1396,6 +1422,7 @@ const std::vector<EidosMethodSignature_CSP> *Community_Class::Methods(void) cons
 		methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gStr_simulationFinished, kEidosValueMaskVOID)));
 		methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gStr_speciesWithIDs, kEidosValueMaskObject, gSLiM_Species_Class))->AddInt("ids"));
 		methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gStr_subpopulationsWithIDs, kEidosValueMaskObject, gSLiM_Subpopulation_Class))->AddInt("ids"));
+		methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gStr_subpopulationsWithNames, kEidosValueMaskObject, gSLiM_Subpopulation_Class))->AddString("names"));
 		methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gEidosStr_usage, kEidosValueMaskFloat | kEidosValueMaskSingleton)));
 		
 		std::sort(methods->begin(), methods->end(), CompareEidosCallSignatures);
