@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 12/13/14.
-//  Copyright (c) 2014-2023 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2014-2024 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -60,7 +60,7 @@ MutationType::MutationType(Species &p_species, slim_objectid_t p_mutation_type_i
 #else
 MutationType::MutationType(Species &p_species, slim_objectid_t p_mutation_type_id, double p_dominance_coeff, bool p_nuc_based, DFEType p_dfe_type, std::vector<double> p_dfe_parameters, std::vector<std::string> p_dfe_strings) :
 #endif
-self_symbol_(EidosStringRegistry::GlobalStringIDForString(SLiMEidosScript::IDStringWithPrefix('m', p_mutation_type_id)), EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Object_singleton(this, gSLiM_MutationType_Class))),
+self_symbol_(EidosStringRegistry::GlobalStringIDForString(SLiMEidosScript::IDStringWithPrefix('m', p_mutation_type_id)), EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Object(this, gSLiM_MutationType_Class))),
 	species_(p_species), mutation_type_id_(p_mutation_type_id), dominance_coeff_(static_cast<slim_selcoeff_t>(p_dominance_coeff)), haploid_dominance_coeff_(1.0), dfe_type_(p_dfe_type), dfe_parameters_(std::move(p_dfe_parameters)), dfe_strings_(std::move(p_dfe_strings)), nucleotide_based_(p_nuc_based), convert_to_substitution_(false), stack_policy_(MutationStackPolicy::kStack), stack_group_(p_mutation_type_id), cached_dfe_script_(nullptr)
 #ifdef SLIM_KEEP_MUTTYPE_REGISTRIES
 	, muttype_registry_call_count_(0), keeping_muttype_registry_(false)
@@ -69,6 +69,9 @@ self_symbol_(EidosStringRegistry::GlobalStringIDForString(SLiMEidosScript::IDStr
 	, mutation_type_index_(p_mutation_type_index)
 #endif
 {
+	// self_symbol_ is always a constant, but can't be marked as such on construction
+	self_symbol_.second->MarkAsConstant();
+	
 	// In WF models, convertToSubstitution defaults to T; in nonWF models it defaults to F as specified above
 	if (species_.community_.ModelType() == SLiMModelType::kModelTypeWF)
 		convert_to_substitution_ = true;
@@ -170,14 +173,14 @@ void MutationType::ParseDFEParameters(std::string &p_dfe_type_string, const Eido
 			if ((dfe_param_type != EidosValueType::kValueFloat) && (dfe_param_type != EidosValueType::kValueInt))
 				EIDOS_TERMINATION << "ERROR (MutationType::ParseDFEParameters): the parameters for a DFE of type '" << *p_dfe_type << "' must be of type numeric (integer or float)." << EidosTerminate();
 			
-			p_dfe_parameters->emplace_back(dfe_param_value->FloatAtIndex(0, nullptr));
+			p_dfe_parameters->emplace_back(dfe_param_value->NumericAtIndex_NOCAST(0, nullptr));
 		}
 		else
 		{
 			if (dfe_param_type != EidosValueType::kValueString)
 				EIDOS_TERMINATION << "ERROR (MutationType::ParseDFEParameters): the parameters for a DFE of type '" << *p_dfe_type << "' must be of type string." << EidosTerminate();
 			
-			p_dfe_strings->emplace_back(dfe_param_value->StringAtIndex(0, nullptr));
+			p_dfe_strings->emplace_back(dfe_param_value->StringAtIndex_NOCAST(0, nullptr));
 		}
 	}
 	
@@ -325,9 +328,9 @@ double MutationType::DrawSelectionCoefficient(void) const
 				int result_count = result->Count();
 				
 				if ((result_type == EidosValueType::kValueFloat) && (result_count == 1))
-					sel_coeff = result->FloatAtIndex(0, nullptr);
+					sel_coeff = result->FloatData()[0];
 				else if ((result_type == EidosValueType::kValueInt) && (result_count == 1))
-					sel_coeff = result->IntAtIndex(0, nullptr);
+					sel_coeff = result->IntData()[0];
 				else
 					EIDOS_TERMINATION << "ERROR (MutationType::DrawSelectionCoefficient): type 's' DFE callbacks must provide a singleton float or integer return value." << EidosTerminate(nullptr);
 			}
@@ -418,7 +421,7 @@ EidosValue_SP MutationType::GetProperty(EidosGlobalStringID p_property_id)
 		case gID_id:						// ACCELERATED
 		{
 			if (!cached_value_muttype_id_)
-				cached_value_muttype_id_ = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int_singleton(mutation_type_id_));
+				cached_value_muttype_id_ = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int(mutation_type_id_));
 			return cached_value_muttype_id_;
 		}
 		case gID_distributionType:
@@ -435,13 +438,13 @@ EidosValue_SP MutationType::GetProperty(EidosGlobalStringID p_property_id)
 			{
 				if (!static_dfe_string_f)
 				{
-					static_dfe_string_f = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(gStr_f));
-					static_dfe_string_g = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(gStr_g));
-					static_dfe_string_e = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(gStr_e));
-					static_dfe_string_n = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(gEidosStr_n));
-					static_dfe_string_w = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(gStr_w));
-					static_dfe_string_p = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(gStr_p));
-					static_dfe_string_s = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(gEidosStr_s));
+					static_dfe_string_f = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(gStr_f));
+					static_dfe_string_g = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(gStr_g));
+					static_dfe_string_e = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(gStr_e));
+					static_dfe_string_n = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(gEidosStr_n));
+					static_dfe_string_w = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(gStr_w));
+					static_dfe_string_p = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(gStr_p));
+					static_dfe_string_s = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(gEidosStr_s));
 				}
 			}
 			
@@ -460,28 +463,28 @@ EidosValue_SP MutationType::GetProperty(EidosGlobalStringID p_property_id)
 		case gID_distributionParams:
 		{
 			if (dfe_parameters_.size() > 0)
-				return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Float_vector(dfe_parameters_));
+				return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Float(dfe_parameters_));
 			else
-				return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_vector(dfe_strings_));
+				return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(dfe_strings_));
 		}
 		case gID_species:
 		{
-			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Object_singleton(&species_, gSLiM_Species_Class));
+			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Object(&species_, gSLiM_Species_Class));
 		}
 			
 			// variables
 		case gEidosID_color:
-			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(color_));
+			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(color_));
 		case gID_colorSubstitution:
-			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(color_sub_));
+			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(color_sub_));
 		case gID_convertToSubstitution:
 			return (convert_to_substitution_ ? gStaticEidosValue_LogicalT : gStaticEidosValue_LogicalF);
 		case gID_dominanceCoeff:			// ACCELERATED
-			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Float_singleton(dominance_coeff_));
+			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Float(dominance_coeff_));
 		case gID_haploidDominanceCoeff:
-			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Float_singleton(haploid_dominance_coeff_));
+			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Float(haploid_dominance_coeff_));
 		case gID_mutationStackGroup:
-			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int_singleton(stack_group_));
+			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int(stack_group_));
 		case gID_nucleotideBased:
 			return (nucleotide_based_ ? gStaticEidosValue_LogicalT : gStaticEidosValue_LogicalF);
 		case gID_mutationStackPolicy:
@@ -496,9 +499,9 @@ EidosValue_SP MutationType::GetProperty(EidosGlobalStringID p_property_id)
 				{
 					THREAD_SAFETY_IN_ACTIVE_PARALLEL("MutationType::GetProperty(): usage of statics");
 					
-					static_policy_string_s = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(gEidosStr_s));
-					static_policy_string_f = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(gStr_f));
-					static_policy_string_l = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(gStr_l));
+					static_policy_string_s = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(gEidosStr_s));
+					static_policy_string_f = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(gStr_f));
+					static_policy_string_l = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(gStr_l));
 				}
 			}
 			
@@ -517,7 +520,7 @@ EidosValue_SP MutationType::GetProperty(EidosGlobalStringID p_property_id)
 			if (tag_value == SLIM_TAG_UNSET_VALUE)
 				EIDOS_TERMINATION << "ERROR (MutationType::GetProperty): property tag accessed on mutation type before being set." << EidosTerminate();
 			
-			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int_singleton(tag_value));
+			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int(tag_value));
 		}
 			
 			// all others, including gID_none
@@ -528,7 +531,7 @@ EidosValue_SP MutationType::GetProperty(EidosGlobalStringID p_property_id)
 
 EidosValue *MutationType::GetProperty_Accelerated_id(EidosObject **p_values, size_t p_values_size)
 {
-	EidosValue_Int_vector *int_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int_vector())->resize_no_initialize(p_values_size);
+	EidosValue_Int *int_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int())->resize_no_initialize(p_values_size);
 	
 	for (size_t value_index = 0; value_index < p_values_size; ++value_index)
 	{
@@ -542,7 +545,7 @@ EidosValue *MutationType::GetProperty_Accelerated_id(EidosObject **p_values, siz
 
 EidosValue *MutationType::GetProperty_Accelerated_tag(EidosObject **p_values, size_t p_values_size)
 {
-	EidosValue_Int_vector *int_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int_vector())->resize_no_initialize(p_values_size);
+	EidosValue_Int *int_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int())->resize_no_initialize(p_values_size);
 	
 	for (size_t value_index = 0; value_index < p_values_size; ++value_index)
 	{
@@ -560,7 +563,7 @@ EidosValue *MutationType::GetProperty_Accelerated_tag(EidosObject **p_values, si
 
 EidosValue *MutationType::GetProperty_Accelerated_dominanceCoeff(EidosObject **p_values, size_t p_values_size)
 {
-	EidosValue_Float_vector *float_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Float_vector())->resize_no_initialize(p_values_size);
+	EidosValue_Float *float_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Float())->resize_no_initialize(p_values_size);
 	
 	for (size_t value_index = 0; value_index < p_values_size; ++value_index)
 	{
@@ -579,7 +582,7 @@ void MutationType::SetProperty(EidosGlobalStringID p_property_id, const EidosVal
 	{
 		case gEidosID_color:
 		{
-			color_ = p_value.StringAtIndex(0, nullptr);
+			color_ = p_value.StringAtIndex_NOCAST(0, nullptr);
 			if (!color_.empty())
 				Eidos_GetColorComponents(color_, &color_red_, &color_green_, &color_blue_);
 			return;
@@ -587,7 +590,7 @@ void MutationType::SetProperty(EidosGlobalStringID p_property_id, const EidosVal
 			
 		case gID_colorSubstitution:
 		{
-			color_sub_ = p_value.StringAtIndex(0, nullptr);
+			color_sub_ = p_value.StringAtIndex_NOCAST(0, nullptr);
 			if (!color_sub_.empty())
 				Eidos_GetColorComponents(color_sub_, &color_sub_red_, &color_sub_green_, &color_sub_blue_);
 			return;
@@ -595,7 +598,7 @@ void MutationType::SetProperty(EidosGlobalStringID p_property_id, const EidosVal
 			
 		case gID_convertToSubstitution:				// ACCELERATED
 		{
-			eidos_logical_t value = p_value.LogicalAtIndex(0, nullptr);
+			eidos_logical_t value = p_value.LogicalAtIndex_NOCAST(0, nullptr);
 			
 			convert_to_substitution_ = value;
 			return;
@@ -603,7 +606,7 @@ void MutationType::SetProperty(EidosGlobalStringID p_property_id, const EidosVal
 			
 		case gID_dominanceCoeff:
 		{
-			double value = p_value.FloatAtIndex(0, nullptr);
+			double value = p_value.FloatAtIndex_NOCAST(0, nullptr);
 			
 			dominance_coeff_ = static_cast<slim_selcoeff_t>(value);		// intentionally no bounds check
 			
@@ -617,7 +620,7 @@ void MutationType::SetProperty(EidosGlobalStringID p_property_id, const EidosVal
 			
 		case gID_haploidDominanceCoeff:
 		{
-			double value = p_value.FloatAtIndex(0, nullptr);
+			double value = p_value.FloatAtIndex_NOCAST(0, nullptr);
 			
 			haploid_dominance_coeff_ = static_cast<slim_selcoeff_t>(value);		// intentionally no bounds check
 			
@@ -631,7 +634,7 @@ void MutationType::SetProperty(EidosGlobalStringID p_property_id, const EidosVal
 			
 		case gID_mutationStackGroup:
 		{
-			int64_t new_group = p_value.IntAtIndex(0, nullptr);
+			int64_t new_group = p_value.IntAtIndex_NOCAST(0, nullptr);
 			
 			if (nucleotide_based_ && (new_group != -1))
 				EIDOS_TERMINATION << "ERROR (MutationType::SetProperty): property " << EidosStringRegistry::StringForGlobalStringID(p_property_id) << " must be -1 for nucleotide-based mutation types." << EidosTerminate();
@@ -644,7 +647,7 @@ void MutationType::SetProperty(EidosGlobalStringID p_property_id, const EidosVal
 			
 		case gID_mutationStackPolicy:
 		{
-			std::string value = p_value.StringAtIndex(0, nullptr);
+			std::string value = p_value.StringAtIndex_NOCAST(0, nullptr);
 			
 			if (nucleotide_based_ && (value != gStr_l))
 				EIDOS_TERMINATION << "ERROR (MutationType::SetProperty): property " << EidosStringRegistry::StringForGlobalStringID(p_property_id) << " must be 'l' for nucleotide-based mutation types." << EidosTerminate();
@@ -664,7 +667,7 @@ void MutationType::SetProperty(EidosGlobalStringID p_property_id, const EidosVal
 			
 		case gID_tag:				// ACCELERATED
 		{
-			slim_usertag_t value = SLiMCastToUsertagTypeOrRaise(p_value.IntAtIndex(0, nullptr));
+			slim_usertag_t value = SLiMCastToUsertagTypeOrRaise(p_value.IntAtIndex_NOCAST(0, nullptr));
 			
 			tag_value_ = value;
 			return;
@@ -681,14 +684,14 @@ void MutationType::SetProperty_Accelerated_convertToSubstitution(EidosObject **p
 {
 	if (p_source_size == 1)
 	{
-		eidos_logical_t source_value = p_source.LogicalAtIndex(0, nullptr);
+		eidos_logical_t source_value = p_source.LogicalAtIndex_NOCAST(0, nullptr);
 		
 		for (size_t value_index = 0; value_index < p_values_size; ++value_index)
 			((MutationType *)(p_values[value_index]))->convert_to_substitution_ = source_value;
 	}
 	else
 	{
-		const eidos_logical_t *source_data = p_source.LogicalVector()->data();
+		const eidos_logical_t *source_data = p_source.LogicalData();
 		
 		for (size_t value_index = 0; value_index < p_values_size; ++value_index)
 			((MutationType *)(p_values[value_index]))->convert_to_substitution_ = source_data[value_index];
@@ -700,14 +703,14 @@ void MutationType::SetProperty_Accelerated_tag(EidosObject **p_values, size_t p_
 	// SLiMCastToUsertagTypeOrRaise() is a no-op at present
 	if (p_source_size == 1)
 	{
-		int64_t source_value = p_source.IntAtIndex(0, nullptr);
+		int64_t source_value = p_source.IntAtIndex_NOCAST(0, nullptr);
 		
 		for (size_t value_index = 0; value_index < p_values_size; ++value_index)
 			((MutationType *)(p_values[value_index]))->tag_value_ = source_value;
 	}
 	else
 	{
-		const int64_t *source_data = p_source.IntVector()->data();
+		const int64_t *source_data = p_source.IntData();
 		
 		for (size_t value_index = 0; value_index < p_values_size; ++value_index)
 			((MutationType *)(p_values[value_index]))->tag_value_ = source_data[value_index];
@@ -731,23 +734,16 @@ EidosValue_SP MutationType::ExecuteMethod_drawSelectionCoefficient(EidosGlobalSt
 #pragma unused (p_method_id, p_arguments, p_interpreter)
 	EidosValue_SP result_SP(nullptr);
 	EidosValue *n_value = p_arguments[0].get();
-	int64_t num_draws = n_value->IntAtIndex(0, nullptr);
+	int64_t num_draws = n_value->IntAtIndex_NOCAST(0, nullptr);
 	
 	if (num_draws < 0)
 		EIDOS_TERMINATION << "ERROR (ExecuteMethod_drawSelectionCoefficient): drawSelectionCoefficient() requires n to be greater than or equal to 0 (" << num_draws << " supplied)." << EidosTerminate(nullptr);
 	
-	if (num_draws == 1)
-	{
-		result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Float_singleton(DrawSelectionCoefficient()));
-	}
-	else
-	{
-		EidosValue_Float_vector *float_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Float_vector())->resize_no_initialize(num_draws);
-		result_SP = EidosValue_SP(float_result);
-		
-		for (int64_t draw_index = 0; draw_index < num_draws; ++draw_index)
-			float_result->set_float_no_check(DrawSelectionCoefficient(), draw_index);
-	}
+	EidosValue_Float *float_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Float())->resize_no_initialize(num_draws);
+	result_SP = EidosValue_SP(float_result);
+	
+	for (int64_t draw_index = 0; draw_index < num_draws; ++draw_index)
+		float_result->set_float_no_check(DrawSelectionCoefficient(), draw_index);
 	
 	return result_SP;
 }
@@ -758,7 +754,7 @@ EidosValue_SP MutationType::ExecuteMethod_setDistribution(EidosGlobalStringID p_
 {
 #pragma unused (p_method_id, p_arguments, p_interpreter)
 	EidosValue *distributionType_value = p_arguments[0].get();
-	std::string dfe_type_string = distributionType_value->StringAtIndex(0, nullptr);
+	std::string dfe_type_string = distributionType_value->StringAtIndex_NOCAST(0, nullptr);
 	
 	// Parse the DFE type and parameters, and do various sanity checks
 	DFEType dfe_type;
