@@ -44,8 +44,8 @@ class SLiMEidosBlock;
 
 
 // SLiM version: see also Info.plist and QtSLiM.pro
-#define SLIM_VERSION_STRING	("4.2.2")
-#define SLIM_VERSION_FLOAT	(4.22)
+#define SLIM_VERSION_STRING	("4.3")
+#define SLIM_VERSION_FLOAT	(4.3)
 
 
 // This should be called once at startup to give SLiM an opportunity to initialize static state
@@ -865,6 +865,8 @@ extern const std::string &gStr_maxDistance;
 
 extern const std::string &gStr_ancestralNucleotides;
 extern const std::string &gStr_nucleotides;
+extern const std::string &gStr_genomicElementForPosition;
+extern const std::string &gStr_hasGenomicElementForPosition;
 extern const std::string &gStr_setAncestralNucleotides;
 extern const std::string &gStr_setGeneConversion;
 extern const std::string &gStr_setHotspotMap;
@@ -904,6 +906,7 @@ extern const std::string &gStr_mutationTypesWithIDs;
 extern const std::string &gStr_scriptBlocksWithIDs;
 extern const std::string &gStr_speciesWithIDs;
 extern const std::string &gStr_subpopulationsWithIDs;
+extern const std::string &gStr_subpopulationsWithNames;
 extern const std::string &gStr_individualsWithPedigreeIDs;
 extern const std::string &gStr_killIndividuals;
 extern const std::string &gStr_mutationCounts;
@@ -938,6 +941,7 @@ extern const std::string &gStr_treeSeqCoalesced;
 extern const std::string &gStr_treeSeqSimplify;
 extern const std::string &gStr_treeSeqRememberIndividuals;
 extern const std::string &gStr_treeSeqOutput;
+extern const std::string &gStr__debug;	// internal
 extern const std::string &gStr_setMigrationRates;
 extern const std::string &gStr_deviatePositions;
 extern const std::string &gStr_pointDeviated;
@@ -1281,6 +1285,8 @@ enum _SLiMGlobalStringID : int {
 	
 	gID_ancestralNucleotides,
 	gID_nucleotides,
+	gID_genomicElementForPosition,
+	gID_hasGenomicElementForPosition,
 	gID_setAncestralNucleotides,
 	gID_setGeneConversion,
 	gID_setHotspotMap,
@@ -1320,6 +1326,7 @@ enum _SLiMGlobalStringID : int {
 	gID_scriptBlocksWithIDs,
 	gID_speciesWithIDs,
 	gID_subpopulationsWithIDs,
+	gID_subpopulationsWithNames,
 	gID_individualsWithPedigreeIDs,
 	gID_killIndividuals,
 	gID_mutationCounts,
@@ -1354,6 +1361,7 @@ enum _SLiMGlobalStringID : int {
 	gID_treeSeqSimplify,
 	gID_treeSeqRememberIndividuals,
 	gID_treeSeqOutput,
+	gID__debug,		// internal
 	gID_setMigrationRates,
 	gID_deviatePositions,
 	gID_pointDeviated,
