@@ -104,6 +104,11 @@ private:
 	std::ostream &execution_output_;
 	std::ostream &error_output_;
 	
+	// a flag controlling the special use of SLiMUndefinedIdentifierException by
+	// EidosSymbolTable::_GetValue(); see Community::_EvaluateTickRangeNode() for
+	// the use of this facility.
+	bool use_custom_undefined_identifier_raise_ = false;
+	
 	// The standard built-in function map, set up by CacheBuiltInFunctionMap()
 	static EidosFunctionMap *s_built_in_function_map_;
 	
@@ -319,6 +324,10 @@ public:
 		//else
 		//	return EvaluateNode(p_node);
 	}
+	
+	// Support for tick range expression parsing; see Community::_EvaluateTickRangeNode()
+	bool UseCustomUndefinedIdentifierRaise(void) { return use_custom_undefined_identifier_raise_; }
+	void SetUseCustomUndefinedIdentifierRaise(bool p_flag) { use_custom_undefined_identifier_raise_ = p_flag; }
 };
 
 
